TAG = "OF-LongLegFilter"

local vs = [[
attribute vec4 aPosition;
attribute vec4 aTextureCoord;
varying vec2 vTexCoord;

void main()
{
    gl_Position = aPosition;
    vTexCoord = aTextureCoord.xy;
}
]]

local fs = [[
precision mediump float;
varying vec2 vTexCoord;
uniform sampler2D uTexture0;
uniform int uDebug;
uniform int uLandscape;
uniform float uBeginY;
uniform float uEndY;
uniform float uPercent;

void main()
{
    vec2 texCoord = vTexCoord;
    if (uLandscape == 1)
    {
        texCoord.x = vTexCoord.y;
        texCoord.y = 1.0 - vTexCoord.x;
    }
    
    if (uDebug == 1)
    {
        if (texCoord.y < uBeginY || texCoord.y > uEndY)
        {
            gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
        }
        else
        {
            vec2 tc = texCoord;
            if (uLandscape == 1)
            {
                tc.x = 1.0 - texCoord.y;
                tc.y = texCoord.x;
            }
            gl_FragColor = texture2D(uTexture0, tc);
        }
        return;
    }
    
    float d1 = (uEndY - uBeginY) * uPercent / 2.0;
    float d2 = d1;

    if (texCoord.y > uBeginY - d1 && texCoord.y < uEndY + d2)
    {
        texCoord.y = uBeginY + (texCoord.y - uBeginY + d1) * (uEndY - uBeginY) / (uEndY - uBeginY + d1 + d2);
    }
    else if (texCoord.y <= uBeginY - d1)
    {
        texCoord.y = texCoord.y + d1;
    }
    else
    {
        texCoord.y = texCoord.y - d2;
    }

    vec2 tc = texCoord;
    if (uLandscape == 1)
    {
        tc.x = 1.0 - texCoord.y;
        tc.y = texCoord.x;
    }
    gl_FragColor = texture2D(uTexture0, tc);
}
]]

local render_pass = nil
param_map = {
    Percent_Y = { min = 0.0, max = 10.0 }
    }

function paramValue(filter, param)
    percent = filter:floatParam("I")
    if param_map[param] ~= nil and percent > 0.01 then
        return (param_map[param].max - param_map[param].min) * (percent / 100.0) + param_map[param].min
    else
        return filter:floatParam(param)
    end
end

function initParams(context, filter)
    OF_LOGI(TAG, "call initParams")
    filter:insertFloatParam("I", 0, 100, 0)
    filter:insertFloatParam("Begin", 0.0, 1.0, 0.0)
    filter:insertFloatParam("End", 0.0, 1.0, 1.0)
    filter:insertFloatParam("Percent_Y", 0, 100, 0)
    return OF_Result_Success
end

function initRenderer(context, filter)
    OF_LOGI(TAG, "call initRenderer")
    render_pass = context:createCustomShaderPass(vs, fs)
    return OF_Result_Success
end

function teardownRenderer(context, filter)
    OF_LOGI(TAG, "call teardownRenderer")
    context:destroyCustomShaderPass(render_pass)
    return OF_Result_Success
end

function applyRGBA(context, filter, frameData, inTex, outTex, debugTex)
    context:setViewport(0, 0, outTex.width, outTex.height)
    context:setBlend(false)
    
    local render = context:sharedQuadRender()
    
    context:bindFBO(outTex)
    render_pass:use()
    render_pass:setUniformTexture("uTexture0", 0, inTex.textureID, GL_TEXTURE_2D)
    render_pass:setUniform1f("uBeginY", paramValue(filter, "Begin"))
    render_pass:setUniform1f("uEndY", paramValue(filter, "End"))
    render_pass:setUniform1f("uPercent", paramValue(filter, "Percent_Y") / 100.0)
    render_pass:setUniform1i("uDebug", 0);
    render_pass:setUniform1i("uLandscape", context:config().screenOrientation);
    render:draw(render_pass, false)

    if debugTex ~= nil then
        context:bindFBO(debugTex)
        render_pass:use()
        render_pass:setUniformTexture("uTexture0", 0, inTex.textureID, GL_TEXTURE_2D)
        render_pass:setUniform1f("uBeginY", paramValue(filter, "Begin"))
        render_pass:setUniform1f("uEndY", paramValue(filter, "End"))
        render_pass:setUniform1f("uPercent", paramValue(filter, "Percent_Y") / 100.0)
        render_pass:setUniform1i("uDebug", 1);
        render_pass:setUniform1i("uLandscape", context:config().screenOrientation);
        render:draw(render_pass, false)
    end
    
    return OF_Result_Success
end

function requiredFrameData(context, game)
    return { OF_RequiredFrameData_None }
end

function onApplyParams(context, filter)
    OF_LOGI(TAG, "call onApplyParams")
    return OF_Result_Success
end

function readObject(context, filter, archiveIn)
    OF_LOGI(TAG, "call readObject")
    return OF_Result_Success
end

function writeObject(context, filter, archiveOut)
    OF_LOGI(TAG, "call writeObject")
    return OF_Result_Success
end
